@@ -1,32 +1,24 @@
 //
 //  Little cms
-//  Copyright (C) 1998-2001 Marti Maria
+//  Copyright (C) 1998-2005 Marti Maria
 //
-// THIS SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
-// EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
-// WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
+// Permission is hereby granted, free of charge, to any person obtaining 
+// a copy of this software and associated documentation files (the "Software"), 
+// to deal in the Software without restriction, including without limitation 
+// the rights to use, copy, modify, merge, publish, distribute, sublicense, 
+// and/or sell copies of the Software, and to permit persons to whom the Software 
+// is furnished to do so, subject to the following conditions:
 //
-// IN NO EVENT SHALL MARTI MARIA BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
-// INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
-// OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
-// WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF
-// LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
-// OF THIS SOFTWARE.
+// The above copyright notice and this permission notice shall be included in 
+// all copies or substantial portions of the Software.
 //
-//
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Lesser General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
-//
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Lesser General Public License for more details.
-//
-// You should have received a copy of the GNU Lesser General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
+// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
+// THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
+// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE 
+// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION 
+// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION 
+// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 #include "lcms.h"
 
@@ -58,6 +50,13 @@ applied to preview, is the proofing intent. The second intent  identifies the
 transform intent. Input data of any stage is taked as relative colorimetric
 always.
 
+
+NOTES: V4 states than perceptual & saturation intents between mixed v2 & v4 profiles should 
+scale PCS from a black point equal to ZERO in v2 profiles to the reference media black of
+perceptual v4 PCS. Since I found many v2 profiles to be using a perceptual intent with black 
+point not zero at all, I'm implementing that as a black point compensation from whatever 
+black from perceptal intent to the reference media black for v4 profiles.
+
 */
 
 
@@ -75,6 +74,7 @@ int cdecl cmsChooseCnvrt(int Absolute,
                              LPMAT3 ChromaticAdaptationMatrixOut,
 
                 int DoBlackPointCompensation,
+                double AdaptationState,
                  _cmsADJFN *fn1,
                  LPWMAT3 wm, LPWVEC3 wof);
 
@@ -109,75 +109,64 @@ LCMSAPI LPcmsCIExyY LCMSEXPORT cmsD50_xyY(void)
 #pragma warning(disable : 4100 4505)
 #endif
 
-static
-void Rel2AbsCoefs(LPcmsCIEXYZ BlackPoint,
-                  LPcmsCIEXYZ WhitePoint,
-                  LPcmsCIEXYZ Illuminant,
-                  LPMAT3 ChromaticAdaptationMatrix,
-                  LPMAT3 m, LPVEC3 of)
-{
-     MAT3 Chrm;
-            
-       Chrm = *ChromaticAdaptationMatrix;
-       MAT3inverse(&Chrm, m);
-       VEC3init(of, 0, 0, 0);     
-      
-}
 
-// Calculate a, b for Absolute -> Relativ undoing any chromatic adaptation
-// done by the profile
-
-static
-void Abs2RelCoefs(LPcmsCIEXYZ BlackPoint,
-                  LPcmsCIEXYZ WhitePoint,
-                  LPcmsCIEXYZ Illuminant,
-                  LPMAT3 ChromaticAdaptationMatrix,
-                  LPMAT3 m, LPVEC3 of)
-{
-      
-       *m = *ChromaticAdaptationMatrix;
-        VEC3init(of, 0, 0, 0);       
-}
 
 // join scalings to obtain:
 //     relative input to absolute and then to relative output
 
 static
-void Rel2RelStepAbsCoefs(LPcmsCIEXYZ BlackPointIn,
-                      LPcmsCIEXYZ WhitePointIn,
-                      LPcmsCIEXYZ IlluminantIn,
-                      LPMAT3 ChromaticAdaptationMatrixIn,
+void Rel2RelStepAbsCoefs(double AdaptationState,
 
-                      LPcmsCIEXYZ BlackPointOut,
-                      LPcmsCIEXYZ WhitePointOut,
-                      LPcmsCIEXYZ IlluminantOut,
-                      LPMAT3 ChromaticAdaptationMatrixOut,
+                         LPcmsCIEXYZ BlackPointIn,
+                         LPcmsCIEXYZ WhitePointIn,
+                         LPcmsCIEXYZ IlluminantIn,
+                         LPMAT3 ChromaticAdaptationMatrixIn,
 
-                      LPMAT3 m, LPVEC3 of)
+                         LPcmsCIEXYZ BlackPointOut,
+                         LPcmsCIEXYZ WhitePointOut,
+                         LPcmsCIEXYZ IlluminantOut,
+                         LPMAT3 ChromaticAdaptationMatrixOut,
+
+                         LPMAT3 m, LPVEC3 of)
 {
-       MAT3 min, mout;
-       VEC3 ofin, ofout;
-
        
-       Rel2AbsCoefs(BlackPointIn,
-                    WhitePointIn,
-                    IlluminantIn, 
-                    ChromaticAdaptationMatrixIn,
-                    &min, &ofin);
+       VEC3 WtPtIn, WtPtInAdapted;
+       VEC3 WtPtOut, WtPtOutAdapted;
+       MAT3 Scale, m1, m2, m3;
+       
+       VEC3init(&WtPtIn, WhitePointIn->X, WhitePointIn->Y, WhitePointIn->Z);
+       MAT3eval(&WtPtInAdapted, ChromaticAdaptationMatrixIn, &WtPtIn);
+                  
+       VEC3init(&WtPtOut, WhitePointOut->X, WhitePointOut->Y, WhitePointOut->Z);
+       MAT3eval(&WtPtOutAdapted, ChromaticAdaptationMatrixOut, &WtPtOut);
 
-       Abs2RelCoefs(BlackPointOut,
-                    WhitePointOut,
-                    IlluminantOut, 
-                    ChromaticAdaptationMatrixOut,
-                    &mout, &ofout);
+       VEC3init(&Scale.v[0], WtPtInAdapted.n[0] / WtPtOutAdapted.n[0], 0, 0);
+       VEC3init(&Scale.v[1], 0, WtPtInAdapted.n[1] / WtPtOutAdapted.n[1], 0);
+       VEC3init(&Scale.v[2], 0, 0, WtPtInAdapted.n[2] / WtPtOutAdapted.n[2]);
 
 
-       // Chromatic adaptation matrix should be conmutative
-       // min * mout = mout * min
-           
-       MAT3per(m,  &min, &mout);
+       // Adaptation state
+
+       if (AdaptationState == 1.0) {
+
+           // Observer is fully adapted. Keep chromatic adaptation 
+
+           CopyMemory(m, &Scale, sizeof(MAT3));
+
+       }
+       else {
+
+            // Observer is not adapted, undo the chromatic adaptation
+            m1 = *ChromaticAdaptationMatrixIn;
+            MAT3inverse(&m1, &m2);
+       
+            MAT3per(&m3, &m2, &Scale);
+            MAT3per(m, &m3, ChromaticAdaptationMatrixOut);
+       }
+
+            
        VEC3init(of, 0.0, 0.0, 0.0);
-              
+                    
 }
 
 
@@ -235,6 +224,24 @@ void ComputeBlackPointCompensationFactors(LPcmsCIEXYZ BlackPointIn,
 
 }
 
+// Return TRUE if both m and of are empy -- "m" being identity and "of" being 0
+
+static
+BOOL IdentityParameters(LPWMAT3 m, LPWVEC3 of)
+{   
+    WVEC3 wv0;
+
+    VEC3initF(&wv0, 0, 0, 0);
+
+    if (!MAT3isIdentity(m, 0.00001)) return FALSE;
+    if (!VEC3equal(of, &wv0, 0.00001)) return FALSE;
+
+    return TRUE;
+}
+
+
+
+
 // ----------------------------------------- Inter PCS conversions
 
 // XYZ to XYZ linear scaling. Aso used on Black point compensation
@@ -251,9 +258,9 @@ void XYZ2XYZ(WORD In[], WORD Out[], LPWMAT3 m, LPWVEC3 of)
 
     MAT3evalW(&r, m, &a);
 
-    Out[0] = Clamp_XYZ((r.n[VX] + of->n[VX]) >> 1);
-    Out[1] = Clamp_XYZ((r.n[VY] + of->n[VY]) >> 1);
-    Out[2] = Clamp_XYZ((r.n[VZ] + of->n[VZ]) >> 1);
+    Out[0] = _cmsClampWord((r.n[VX] + of->n[VX]) >> 1);
+    Out[1] = _cmsClampWord((r.n[VY] + of->n[VY]) >> 1);
+    Out[2] = _cmsClampWord((r.n[VZ] + of->n[VZ]) >> 1);
 }
 
 
@@ -308,6 +315,7 @@ int FromXYZRelLUT(int Absolute,
                              LPMAT3 ChromaticAdaptationMatrixOut,
 
                  int DoBlackPointCompensation,
+                 double AdaptationState,
                  _cmsADJFN *fn1,
                  LPMAT3 m, LPVEC3 of)
 
@@ -323,15 +331,16 @@ int FromXYZRelLUT(int Absolute,
                                    // From input relative to absolute, and then
                                    // back to output relative
 
-                                   Rel2RelStepAbsCoefs(BlackPointIn,
-                                                  WhitePointIn,
-                                                  IlluminantIn,
-                                                  ChromaticAdaptationMatrixIn,
-                                                  BlackPointOut,
-                                                  WhitePointOut,
-                                                  IlluminantOut,
-                                                  ChromaticAdaptationMatrixOut,
-                                                  m, of);
+                                   Rel2RelStepAbsCoefs(AdaptationState,
+                                                       BlackPointIn,
+                                                       WhitePointIn,
+                                                       IlluminantIn,
+                                                       ChromaticAdaptationMatrixIn,
+                                                       BlackPointOut,
+                                                       WhitePointOut,
+                                                       IlluminantOut,
+                                                       ChromaticAdaptationMatrixOut,
+                                                       m, of);
                                    *fn1 = XYZ2XYZ;
 
                             }
@@ -354,18 +363,7 @@ int FromXYZRelLUT(int Absolute,
                             }
                             break;
 
-                     // From relative XYZ to absolute XYZ. 
-
-                     case XYZAbs:
-
-                            Rel2AbsCoefs(BlackPointIn, 
-                                         WhitePointIn, 
-                                         IlluminantIn, 
-                                         ChromaticAdaptationMatrixIn,
-                                         m, of);
-                            *fn1 = XYZ2XYZ;
-                            break;
-
+                    
                      // From relative XYZ to Relative Lab
 
                      case LabRel:
@@ -377,7 +375,8 @@ int FromXYZRelLUT(int Absolute,
                             if (Absolute)
                             {   
 
-                                Rel2RelStepAbsCoefs(BlackPointIn,
+                                Rel2RelStepAbsCoefs(AdaptationState,
+                                                    BlackPointIn,
                                                     WhitePointIn,
                                                     IlluminantIn,
                                                     ChromaticAdaptationMatrixIn,
@@ -411,25 +410,7 @@ int FromXYZRelLUT(int Absolute,
                             }
                             break;
 
-                     // From relative XYZ To Absolute Lab, adjusting to D50
-
-                     case LabAbs:
-                                {
-                                Rel2RelStepAbsCoefs(BlackPointIn,
-                                                    WhitePointIn,
-                                                    IlluminantIn,
-                                                    ChromaticAdaptationMatrixIn,
-                                                    BlackPointOut,
-                                                    WhitePointOut,
-                                                    IlluminantOut,
-                                                    ChromaticAdaptationMatrixOut,
-                                                    m, of);
-                                
-                                *fn1 = XYZ2Lab;
-                                }
-                            break;
-
-
+                    
                      default: return FALSE;
                      }
 
@@ -437,49 +418,6 @@ int FromXYZRelLUT(int Absolute,
 }
 
 
-// Since XYZ comes in absolute colorimetry, no endpoints on input
-// are needed.
-
-static
-int FromXYZAbsLUT(
-                 int Phase2, LPcmsCIEXYZ BlackPointOut,
-                             LPcmsCIEXYZ WhitePointOut,
-                             LPcmsCIEXYZ IlluminantOut,
-                             LPMAT3 ChromaticAdaptationMatrixOut,
-
-                 _cmsADJFN *fn1,
-                 LPMAT3 m, LPVEC3 of)
-
-{
-
-          switch (Phase2) {
-
-              case XYZRel:
-                     Abs2RelCoefs(BlackPointOut, 
-                                  WhitePointOut, IlluminantOut, 
-                                  ChromaticAdaptationMatrixOut, m, of);
-                     *fn1 = XYZ2XYZ;
-                     break;
-
-              case XYZAbs:         // Identity
-                     *fn1 = NULL;
-                     break;
-
-              case LabRel:
-                     Abs2RelCoefs(BlackPointOut, 
-                                  WhitePointOut, IlluminantOut, 
-                                  ChromaticAdaptationMatrixOut, m, of);
-                     *fn1 = XYZ2Lab;
-                     break;
-
-              case LabAbs:
-                     *fn1 = XYZ2Lab;
-                     break;
-
-              default: return FALSE;
-              }
-       return TRUE;
-}
 
 
 // From Lab Relative type LUT
@@ -497,6 +435,8 @@ int FromLabRelLUT(int Absolute,
                              LPMAT3 ChromaticAdaptationMatrixOut,
 
                 int DoBlackPointCompensation,
+                double AdaptationState,
+
                  _cmsADJFN *fn1,
                  LPMAT3 m, LPVEC3 of)
 {
@@ -512,15 +452,16 @@ int FromLabRelLUT(int Absolute,
                             // From lab relative, to XYZ absolute, and then,
                             // back to XYZ relative
 
-                            Rel2RelStepAbsCoefs(BlackPointIn,
-                                           WhitePointIn,
-                                           cmsD50_XYZ(),
-                                           ChromaticAdaptationMatrixIn,
-                                           BlackPointOut,
-                                           WhitePointOut,
-                                           IlluminantOut,
-                                           ChromaticAdaptationMatrixOut,
-                                           m, of);
+                            Rel2RelStepAbsCoefs(AdaptationState,
+                                                BlackPointIn,
+                                                WhitePointIn,
+                                                cmsD50_XYZ(),
+                                                ChromaticAdaptationMatrixIn,
+                                                BlackPointOut,
+                                                WhitePointOut,
+                                                IlluminantOut,
+                                                ChromaticAdaptationMatrixOut,
+                                                m, of);
 
                             *fn1 = Lab2XYZ;
 
@@ -545,37 +486,24 @@ int FromLabRelLUT(int Absolute,
                      break;
 
 
-              // From Relative Lab to XYZ absolute. First covert to relative XYZ,
-              // then to absolute XYZ
-
-              case XYZAbs: {
-
-                     Rel2AbsCoefs(BlackPointIn, 
-                                  WhitePointIn, 
-                                  cmsD50_XYZ(), 
-                                  ChromaticAdaptationMatrixIn,
-                                  m, of);
-
-                     *fn1 = Lab2XYZ;
-                     }
-                     break;
 
               case LabRel:
 
                      if (Absolute) {
 
-                     // First pass to XYZ using the input illuminant
-                     // * InIlluminant / D50, then to absolute. Then
-                     // to relative, but for input
+                             // First pass to XYZ using the input illuminant
+                             // * InIlluminant / D50, then to absolute. Then
+                             // to relative, but for input
 
-                     Rel2RelStepAbsCoefs(BlackPointIn, 
-                                         WhitePointIn, IlluminantIn,
-                                         ChromaticAdaptationMatrixIn,
-                                         BlackPointOut, 
-                                         WhitePointOut, cmsD50_XYZ(),
-                                         ChromaticAdaptationMatrixOut,
-                                         m, of);
-                     *fn1 = Lab2XYZ2Lab;
+                             Rel2RelStepAbsCoefs(AdaptationState,
+                                                 BlackPointIn, 
+                                                 WhitePointIn, IlluminantIn,
+                                                 ChromaticAdaptationMatrixIn,
+                                                 BlackPointOut, 
+                                                 WhitePointOut, cmsD50_XYZ(),
+                                                 ChromaticAdaptationMatrixOut,
+                                                 m, of);
+                             *fn1 = Lab2XYZ2Lab;
                      }
                      else
                      {      // Lab -> Lab relative don't need any adjust unless
@@ -598,11 +526,6 @@ int FromLabRelLUT(int Absolute,
                      }
                      break;
 
-              case LabAbs:
-                     Rel2AbsCoefs(BlackPointIn, WhitePointIn, cmsD50_XYZ(), 
-                         ChromaticAdaptationMatrixIn, m, of);
-                     *fn1 = Lab2XYZ2Lab;
-                     break;
 
               default: return FALSE;
               }
@@ -611,56 +534,6 @@ int FromLabRelLUT(int Absolute,
 }
 
 
-// From Lab Absolute LUT, always absolute
-
-static
-int FromLabAbsLUT(           LPcmsCIEXYZ BlackPointIn,
-                             LPcmsCIEXYZ WhitePointIn,
-                             LPcmsCIEXYZ IlluminantIn,
-                             LPMAT3 ChromaticAdaptationMatrixIn,
-
-                 int Phase2, LPcmsCIEXYZ BlackPointOut,
-                             LPcmsCIEXYZ WhitePointOut,
-                             LPcmsCIEXYZ IlluminantOut,
-                             LPMAT3 ChromaticAdaptationMatrixOut,
-
-                 _cmsADJFN *fn1,
-                 LPMAT3 m, LPVEC3 of)
-{
-
-
-          switch (Phase2) {
-
-              
-              case XYZRel:                     
-                     *fn1 = Lab2XYZ;
-                     break;
-
-              case XYZAbs:
-                     Abs2RelCoefs(BlackPointOut, 
-                                  WhitePointOut, IlluminantOut, 
-                                  ChromaticAdaptationMatrixOut, m, of);
-                     *fn1 = Lab2XYZ;
-                     break;
-
-              case LabRel:
-                     Abs2RelCoefs(BlackPointOut, 
-                                  WhitePointOut, IlluminantOut, 
-                                  ChromaticAdaptationMatrixOut, m, of);
-                     *fn1 = Lab2XYZ2Lab;
-                     break;
-
-              case LabAbs:
-                     *fn1 = NULL;
-                     break;
-
-              default: return FALSE;
-              }
-        
-        return TRUE;
-}
-
-
 // This function does calculate the necessary conversion operations
 // needed from transpassing data from a LUT to a LUT. The conversion
 // is modeled as a pointer of function and two coefficients, a and b
@@ -684,6 +557,7 @@ int cmsChooseCnvrt(int Absolute,
                               LPMAT3 ChromaticAdaptationMatrixOut,
 
                   int DoBlackPointCompensation,
+                  double AdaptationState,
                   _cmsADJFN *fn1,
                   LPWMAT3 wm, LPWVEC3 wof)
 {
@@ -711,20 +585,11 @@ int cmsChooseCnvrt(int Absolute,
                                           IlluminantOut,
                                           ChromaticAdaptationMatrixOut,
                                           DoBlackPointCompensation,
+                                          AdaptationState,
                                           fn1, &m, &of);
                      break;
 
        
-       // Input LUT is giving XYZ Absolute values. 
-
-       case XYZAbs:  rc = FromXYZAbsLUT(Phase2,
-                                          BlackPointOut,
-                                          WhitePointOut,
-                                          IlluminantOut,
-                                          ChromaticAdaptationMatrixOut,
-                                          fn1, &m, &of);
-                     break;
-       
 
        // Input LUT is giving Lab relative values
 
@@ -739,24 +604,12 @@ int cmsChooseCnvrt(int Absolute,
                                           IlluminantOut,
                                           ChromaticAdaptationMatrixOut,
                                           DoBlackPointCompensation,
+                                          AdaptationState,
                                           fn1, &m, &of);
                      break;
 
+
        
-       // Input LUT is giving absolute Lab values.
-        
-       case LabAbs:  rc = FromLabAbsLUT(BlackPointIn,
-                                          WhitePointIn,
-                                          IlluminantIn,
-                                          ChromaticAdaptationMatrixIn,
-                                          Phase2,
-                                          BlackPointOut,
-                                          WhitePointOut,
-                                          IlluminantOut,
-                                          ChromaticAdaptationMatrixOut,
-                                          fn1, &m, &of);
-                     break;
-        
 
        // Unrecognized combination
 
@@ -768,5 +621,17 @@ int cmsChooseCnvrt(int Absolute,
        MAT3toFix(wm, &m);
        VEC3toFix(wof, &of);
 
+       // Do some optimization -- discard conversion if identity parameters.
+
+       if (*fn1 == XYZ2XYZ || *fn1 == Lab2XYZ2Lab) {
+
+           if (IdentityParameters(wm, wof))
+               *fn1 = NULL;
+       }
+
+       
        return rc;
 }
+
+
+      
